GDN = {}

-- constants
GDN.name = "GroupDeathNotifier"
GDN.wingedTwilight = "Winged Twilight"
GDN.twilightTormentor = "Twilight Tormentor"
GDN.twilightMatriarch = "Twilight Matriarch"
GDN.unstableFamiliar = "Unstable Familiar"
GDN.unstableClannfear = "Unstable Clannfear"
GDN.volatileFamiliar = "Volatile Familiar"
GDN.feralGuardian = "Feral Guardian"
GDN.eternalGuardian = "Eternal Guardian"
GDN.wildGuardian = "Wild Guardian"
GDN.ROLE_NAMES = {
    [LFG_ROLE_DPS] = "DPS",
    [LFG_ROLE_HEAL] = "Healer",
    [LFG_ROLE_TANK] = "Tank"
}
GDN.roleIcons = {
    ["DPS"] = "/esoui/art/lfg/lfg_icon_dps.dds",
    ["Healer"] = "/esoui/art/lfg/lfg_icon_healer.dds",
    ["Tank"] = "/esoui/art/lfg/lfg_icon_tank.dds",
}
GDN.abilityMap = {
    --Winged Twilight
    [24613] = GDN.wingedTwilight, 
    [24739] = GDN.wingedTwilight, 
    [30581] = GDN.wingedTwilight, 
    [30584] = GDN.wingedTwilight, 
    [30587] = GDN.wingedTwilight,
    --Twilight Tormentor
    [24636] = GDN.twilightTormentor, 
    [30592] = GDN.twilightTormentor, 
    [30595] = GDN.twilightTormentor,
    [30598] = GDN.twilightTormentor,
    --Twilight Matriarch
    [24639] = GDN.twilightMatriarch,
    [30618] = GDN.twilightMatriarch,
    [30622] = GDN.twilightMatriarch,
    [30626] = GDN.twilightMatriarch,
    --Unstable Familiar
    [23304] = GDN.unstableFamiliar,
    [30631] = GDN.unstableFamiliar,
    [30636] = GDN.unstableFamiliar,
    [30641] = GDN.unstableFamiliar,
    --Unstable Clannfear
    [23319] = GDN.unstableClannfear,
    [30647] = GDN.unstableClannfear,
    [30652] = GDN.unstableClannfear,
    [30657] = GDN.unstableClannfear,
    --Volatile Familiar
    [30664] = GDN.volatileFamiliar,
    [30669] = GDN.volatileFamiliar,
    [30674] = GDN.volatileFamiliar,
    [23316] = GDN.volatileFamiliar,
    --Feral Guardian
    [85982] = GDN.feralGuardian,
    [85983] = GDN.feralGuardian,
    [85984] = GDN.feralGuardian,
    [85985] = GDN.feralGuardian,
    --Eternal Guardian
    [85986] = GDN.eternalGuardian,
    [85987] = GDN.eternalGuardian,
    [85988] = GDN.eternalGuardian,
    [85989] = GDN.eternalGuardian,
    --Wild Guardian
    [85990] = GDN.wildGuardian,
    [85991] = GDN.wildGuardian,
    [85992] = GDN.wildGuardian,
    [85993] = GDN.wildGuardian
}

-- functional vars
GDN.savedVariables = {}
GDN.hidden = true
GDN.initialized = false
GDN.deathAnimation = nil
GDN.deathAnimationTimeline = nil
GDN.rezAnimation = nil
GDN.rezAnimationTimeline = nil
GDN.petAnimation = nil
GDN.petAnimationTimeline = nil 

-- default configuration values
function GDN:SetDefaultVars()
    self.animationDuration = 4000
    self.fontScale = 1
    self.useAccountName = false
    self.fullPetName = false
    self.enableRezNotifications = true
    self.enableDeathNotifications = true
    self.enablePetNotifications = true
    self.deathNotificationColor = {
        ["r"] = 1,
        ["g"] = 1,
        ["b"] = 0,
    }
    self.rezNotificationColor = {
        ["r"] = 0,
        ["g"] = 1,
        ["b"] = 0,
    }
    self.petNotificationColor = {
        ["r"] = 1,
        ["g"] = 1,
        ["b"] = 0,
    }
    self.dungeonSettings = {
        ["DPS"] = true,
        ["Healer"] = true,
        ["Tank"] = true,
        ["Pet"] = true
    }
    self.trialSettings = {
        ["DPS"] = true,
        ["Healer"] = true,
        ["Tank"] = true,
        ["Pet"] = true
    }
end

-- initialize saved animations
function GDN:InitAnimations()
    GDN.deathAnimation, GDN.deathAnimationTimeline = GDN.SetUpAnimation(GroupDeathIndicator, GDN.animationDuration)
    GDN.rezAnimation, GDN.rezAnimationTimeline = GDN.SetUpAnimation(GroupRezIndicator, GDN.animationDuration)
    GDN.petAnimation, GDN.petAnimationTimeline = GDN.SetUpAnimation(PetDeathIndicator, GDN.animationDuration)
end

-- init logic
function GDN:Initialize()
    self:SetDefaultVars()
    self.savedVariables = ZO_SavedVars:New("GroupDeathNotifierSavedVariables", 1, nil, {})
    self:RestoreSavedValues()

    if GDN.deathAnimation == nil then
        self:InitAnimations()
    end

    EVENT_MANAGER:RegisterForEvent(GDN.name, EVENT_UNIT_DEATH_STATE_CHANGED, GDN.OnUnitDeathStateChanged)
    EVENT_MANAGER:AddFilterForEvent(GDN.name, EVENT_UNIT_DEATH_STATE_CHANGED, REGISTER_FILTER_UNIT_TAG_PREFIX, "group")

    EVENT_MANAGER:RegisterForEvent(GDN.name, EVENT_EFFECT_CHANGED, GDN.onEffectChanged)
    EVENT_MANAGER:AddFilterForEvent(GDN.name, EVENT_EFFECT_CHANGED, REGISTER_FILTER_UNIT_TAG, "player")

    self.initialized = true
    GDNM.Init()
end

-- show/hide the notification panels to be able to move them
function GDN.ToggleHidden()
    GroupDeathIndicator:SetAlpha(1.0)
    GroupDeathLabel:SetColor(GDN.deathNotificationColor["r"], GDN.deathNotificationColor["g"], GDN.deathNotificationColor["b"])
    GroupDeathIndicator:SetScale(GDN.fontScale)

    GroupRezIndicator:SetAlpha(1.0)
    GroupRezLabel:SetColor(GDN.rezNotificationColor["r"], GDN.rezNotificationColor["g"], GDN.rezNotificationColor["b"])
    GroupRezIndicator:SetScale(GDN.fontScale)

    PetDeathIndicator:SetAlpha(1.0)
    PetDeathLabel:SetColor(GDN.petNotificationColor["r"], GDN.petNotificationColor["g"], GDN.petNotificationColor["b"])
    PetDeathIndicator:SetScale(GDN.fontScale)

    GDN.hidden = not GDN.hidden

    GroupDeathIndicatorIcon:SetHidden(GDN.hidden)
    GroupRezIndicatorIcon:SetHidden(GDN.hidden)
    PetDeathIndicatorIcon:SetHidden(GDN.hidden)
    
    GroupDeathIndicator:SetHidden(GDN.hidden)
    GroupRezIndicator:SetHidden(GDN.hidden)
    PetDeathIndicator:SetHidden(GDN.hidden)

    GroupDeathIndicator:SetMovable(not GDN.hidden)
    GroupRezIndicator:SetMovable(not GDN.hidden)
    PetDeathIndicator:SetMovable(not GDN.hidden)
end

-- handler for when an addon is loaded
function GDN.OnAddOnLoaded(event, addonName)
    if addonName == GDN.name then
        GDN:Initialize()
    end
end

-- handle saving death indicator position to saved vars
function GDN.OnDeathIndicatorMoveStop()
    GDN.savedVariables.deathLeft = GroupDeathIndicator:GetLeft()
    GDN.savedVariables.deathTop = GroupDeathIndicator:GetTop()
end

-- handle saving rez indicator position to saved vars
function GDN.OnRezIndicatorMoveStop()
    GDN.savedVariables.rezLeft = GroupRezIndicator:GetLeft()
    GDN.savedVariables.rezTop = GroupRezIndicator:GetTop()
end

function GDN.OnPetDeathIndicatorMoveStop()
    GDN.savedVariables.petLeft = PetDeathIndicator:GetLeft()
    GDN.savedVariables.petTop = PetDeathIndicator:GetTop()
end

-- restore notification panels to saved positions and sync saved vars with addon
function GDN:RestoreSavedValues()
    if self.savedVariables.deathLeft ~= nil then
        GroupDeathIndicator:ClearAnchors()
        GroupDeathIndicator:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, self.savedVariables.deathLeft, self.savedVariables.deathTop)
    end

    if self.savedVariables.rezLeft ~= nil then
        GroupRezIndicator:ClearAnchors()
        GroupRezIndicator:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, self.savedVariables.rezLeft, self.savedVariables.rezTop)
    end

    if self.savedVariables.petLeft ~= nil then
        PetDeathIndicator:ClearAnchors()
        PetDeathIndicator:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, self.savedVariables.petLeft, self.savedVariables.petTop)
    end

    if self.savedVariables.animationDuration ~= nil then
        GDN.animationDuration = self.savedVariables.animationDuration
        self:InitAnimations()
    end

    if self.savedVariables.deathNotificationColor ~= nil then
        GDN.deathNotificationColor = self.savedVariables.deathNotificationColor
    end

    if self.savedVariables.rezNotificationColor ~= nil then
        GDN.rezNotificationColor = self.savedVariables.rezNotificationColor
    end

    if self.savedVariables.petNotificationColor ~= nil then
        GDN.petNotificationColor = self.savedVariables.petNotificationColor
    end

    if self.savedVariables.dungeonSettings ~= nil then
        GDN.dungeonSettings = self.savedVariables.dungeonSettings
    end

    if self.savedVariables.trialSettings ~= nil then
        GDN.trialSettings = self.savedVariables.trialSettings
    end

    if self.savedVariables.fontScale ~= nil then
        GDN.fontScale = self.savedVariables.fontScale
    end

    if self.savedVariables.useAccountName ~= nil then
        GDN.useAccountName = self.savedVariables.useAccountName
    end

    if self.savedVariables.fullPetName ~= nil then
        GDN.fullPetName = self.savedVariables.fullPetName
    end

    if self.savedVariables.enableDeathNotifications ~= nil then
        GDN.enableDeathNotifications = self.savedVariables.enableDeathNotifications
    end

    if self.savedVariables.enableRezNotifications ~= nil then
        GDN.enableRezNotifications = self.savedVariables.enableRezNotifications
    end

    if self.savedVariables.enablePetNotifications ~= nil then
        GDN.enablePetNotifications = self.savedVariables.enablePetNotifications
    end
end

-- reset positioning saved values to empty
function GDN:ResetSavedPositionValues()
    self.savedVariables.deathLeft = nil
    self.savedVariables.deathTop = nil
    self.savedVariables.rezLeft = nil
    self.savedVariables.rezTop = nil
    self.savedVariables.petLeft = nil
    self.savedVariables.petTop = nil
end

-- reset all saved variables to empty
function GDN:ResetSavedValues()
    self:ResetSavedPositionValues()
    self.savedVariables.animationDuration = nil
    self.savedVariables.deathNotificationColor = nil
    self.savedVariables.rezNotificationColor = nil
    self.savedVariables.dungeonSettings = nil
    self.savedVariables.trialSettings = nil
    self.savedVariables.fontScale = nil
    self.savedVariables.useAccountName = nil
    self.savedVariables.fullPetName = nil
    self.savedVariables.enableDeathNotifications = nil
    self.savedVariables.enableRezNotifications = nil
    self.savedVariables.enablePetNotifications = nil
end

-- get a unit's name from unitTag
function GDN.GetDisplayName(unitTag)
    if GDN.useAccountName then
        return GetUnitDisplayName(unitTag)
    else
        return GetUnitName(unitTag)
    end
end

-- get the name of the unit's role, or empty string if none
function GDN.GetRoleName(unitTag)
    local roleNum = GetGroupMemberAssignedRole(unitTag)
    return roleNum ~= nil and GDN.ROLE_NAMES[roleNum] or ""
end

-- look up config properties based on group size
function GDN.GetGroupProperty(property) 
    local groupSize = GetGroupSize()
    if groupSize == nil or groupSize <= 4 then
        return GDN.dungeonSettings[property]
    else
        return GDN.trialSettings[property]
    end
end

-- get the role icon path for the given unit tag
function GDN.GetRoleIcon(unitTag)
    return GDN.roleIcons[GDN.GetRoleName(unitTag)]
end

-- determine if the unitTag belongs to the player or not
function GDN.IsUnitThePlayer(unitTag) 
    if unitTag == "player" then return true end
    if GetGroupIndexByUnitTag(unitTag) == GetGroupIndexByUnitTag("player") then
        return true
    end
    return false
end

-----------------------------------------------------------------
-- logical check whether or not to show death/rez event
function GDN.ShouldShowEvent(unitTag)
    if GDN.IsUnitThePlayer(unitTag) then return false end

    local roleName = GDN.GetRoleName(unitTag)
    if GDN.GetGroupProperty(roleName) then
        return true
    end
   
    return false
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- methods to check configuration for death/rez notifications
function GDN.ShouldShowDeath(unitTag)
    return GDN.enableDeathNotifications and GDN.ShouldShowEvent(unitTag)
end

function GDN.ShouldShowRez(unitTag)
    return GDN.enableRezNotifications and GDN.ShouldShowEvent(unitTag)
end

function GDN.ShouldShowPetDeath(changeType, abilityId)
    return not IsUnitDead("player") and IsUnitInCombat("player") and GDN.enablePetNotifications and GDN.GetGroupProperty("Pet") and GDN.abilityMap[abilityId] and changeType == 2
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- method to handle player death/rez events
function GDN.OnUnitDeathStateChanged(eventCode, unitTag, isDead)
    if isDead then
        if GDN.ShouldShowDeath(unitTag) then
            if GDN.deathAnimationTimeline ~= nil then GDN.deathAnimationTimeline:Stop() end
            GDN.ShowDeathNotification(unitTag)
        end
    else
        if GDN.ShouldShowRez(unitTag) then
            if GDN.rezAnimationTimeline ~= nil then GDN.rezAnimationTimeline:Stop() end
            GDN.ShowRezNotification(unitTag)
        end
    end
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- handle pet death event
function GDN.onEffectChanged(eventCode, changeType, effectSlot, effectName, unitTag, beginTime, endTime, stackCount, iconName, buffType, effectType, abilityType, statusEffectType, unitName, unitId, abilityId, sourceType)
	if GDN.ShouldShowPetDeath(changeType, abilityId) then
        GDN.ShowPetDeathNotification(abilityId, iconName)
	end
end
-----------------------------------------------------------------


-----------------------------------------------------------------
-- display player death notification
function GDN.ShowDeathNotification(unitTag)
    local userText = GDN.GetDisplayName(unitTag)

    if userText == "" or userText == nil then return end

    local text = userText .. " Died" 
    GroupDeathLabel:SetColor(GDN.deathNotificationColor["r"], GDN.deathNotificationColor["g"], GDN.deathNotificationColor["b"])
    GroupDeathLabel:SetText(text)
    GroupDeathIndicator:SetScale(GDN.fontScale)

    local roleIcon = GDN.GetRoleIcon(unitTag)

    if roleIcon ~= nil then 
        GroupDeathIndicatorIcon:SetTexture(roleIcon) 
        GroupDeathIndicatorIcon:SetHidden(false)
    else
        GroupDeathIndicatorIcon:SetHidden(true)
    end
    
    GDN.AnimateText(GroupDeathIndicator, GDN.deathAnimationTimeline)
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- display player resurrection notification
function GDN.ShowRezNotification(unitTag)
    local userText = GDN.GetDisplayName(unitTag)

    if userText == "" or userText == nil then return end

    local text = userText .. " Resurrected" 
    GroupRezLabel:SetColor(GDN.rezNotificationColor["r"], GDN.rezNotificationColor["g"], GDN.rezNotificationColor["b"])
    GroupRezLabel:SetText(text)
    GroupRezIndicator:SetScale(GDN.fontScale)

    local roleIcon = GDN.GetRoleIcon(unitTag)

    if roleIcon ~= nil then 
        GroupRezIndicatorIcon:SetTexture(roleIcon)
        GroupRezIndicatorIcon:SetHidden(false)
    else
        GroupRezIndicatorIcon:SetHidden(true)
    end

    GDN.AnimateText(GroupRezIndicator, GDN.rezAnimationTimeline)
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- display pet death notification
function GDN.ShowPetDeathNotification(abilityId, iconName)

    local petName

    if GDN.fullPetName then
        petName = GDN.abilityMap[abilityId]
    else
        petName = "Your Pet"
    end

    PetDeathLabel:SetColor(GDN.petNotificationColor["r"], GDN.petNotificationColor["g"], GDN.petNotificationColor["b"])
    PetDeathLabel:SetText(petName .. " Died")
    PetDeathIndicator:SetScale(GDN.fontScale)

    if iconName ~= nil then 
        PetDeathIndicatorIcon:SetTexture(iconName) 
        PetDeathIndicatorIcon:SetHidden(false)
    else
        PetDeathIndicatorIcon:SetHidden(true)
    end
    
    GDN.AnimateText(PetDeathIndicator, GDN.petAnimationTimeline)
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- create reusable animations for text
function GDN.SetUpAnimation(_control, duration)
    local animation, timeline = CreateSimpleAnimation(ANIMATION_ALPHA, _control)
    animation:SetAlphaValues(_control:GetAlpha(), 0)
    animation:SetDuration(duration)
    return animation, timeline
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- animate text fade in/out
function GDN.AnimateText(_control, timeline)
    _control:SetAlpha(1.0)
    _control:SetHidden(false)
    timeline:PlayFromStart()
end
-----------------------------------------------------------------

-----------------------------------------------------------------
-- handler for hiding notifications when menus are open
function GDN.SceneChange(oldState, newState)
    if newState == "hidden" and GDN.hidden then
        GroupDeathIndicator:SetHidden(true)
        GroupRezIndicator:SetHidden(true)
        PetDeathIndicator:SetHidden(true)
    end
end
-----------------------------------------------------------------

-- register scene change handler for hud
local scene = SCENE_MANAGER:GetScene("hud")
scene:RegisterCallback("StateChange", GDN.SceneChange)

-- register addon load event handler
EVENT_MANAGER:RegisterForEvent(GDN.name, EVENT_ADD_ON_LOADED, GDN.OnAddOnLoaded)