GDNM = {}

function GDNM.Init()
    local LAM = LibAddonMenu2
    local saveData = GDN.savedVariables
    local panelName = "GroupDeathNotifierSettingsPanel"
    
    local panelData = {
        type = "panel",
        name = "Group Death Notifier",
        displayName = "Group Death Notifier Settings",
        author = "Fooberticus",
        slashCommand = "/gdn",
        registerForRefresh = true,
    }

    local panel = LAM:RegisterAddonPanel(panelName, panelData)

    local optionsData = {
        [1] = {
            type = "description",
            title = nil,	--(optional)
            text = "Notifies you if a member of your group dies or is resurrected and shows group member role. Also shows when your pet dies. Advanced settings to fully configure notifications are below.",
            width = "full",	--or "half" (optional)
        },
        [2] = {
            type = "header",
            name = "Addon Position",
            width = "full",	--or "half" (optional)
        },
        [3] = {
            type = "description",
            title = nil,	--(optional)
            text = "Click button to show the addon to reposition it, click again to hide. Use the Reset Position button to reset position to default centered position and reload the UI.",
            width = "full",	--or "half" (optional)
        },
        [4] = {
            type = "button",
            name = "Show/Hide Addon",
            tooltip = "Show/Hide the Addon On Screen",
            func = function() GDN.ToggleHidden() end,
            width = "full",	
        },
        [5] = {
            type = "button",
            name = "Reset Position",
            tooltip = "Reset to default position and reload UI",
            func = function() 
                GDN:ResetSavedPositionValues()
                ReloadUI()
            end,
            width = "full",	
            warning = "Will discard position settings and reload the UI!",
        },
        [6] = {
            type = "header",
            name = "Show Notifications",
            width = "full",
        },
        [7] = {
            type = "checkbox",
            name = "Show Death Notifications",
            tooltip = "Turn off to hide ALL death notifications",
            getFunc = function() return GDN.enableDeathNotifications end,
            setFunc = function(value)
                GDN.enableDeathNotifications = value
                saveData.enableDeathNotifications = GDN.enableDeathNotifications
            end,
            width = "full",
        },
        [8] = {
            type = "checkbox",
            name = "Show Rez Notifications",
            tooltip = "Turn off to hide ALL rez notifications",
            getFunc = function() return GDN.enableRezNotifications end,
            setFunc = function(value)
                GDN.enableRezNotifications = value
                saveData.enableRezNotifications = GDN.enableRezNotifications
            end,
            width = "full",
        },
        [9] = {
            type = "checkbox",
            name = "Show Pet Notifications",
            tooltip = "Turn off to hide ALL pet notifications",
            getFunc = function() return GDN.enablePetNotifications end,
            setFunc = function(value)
                GDN.enablePetNotifications = value
                saveData.enablePetNotifications = GDN.enablePetNotifications
            end,
            width = "full",
        },
        [10] = {
            type = "header",
            name = "Notification Appearance",
            width = "full",
        },
        [11] = {
            type = "slider",
            name = "Notification Duration (Seconds)",
            tooltip = "Seconds",
            min = 1,
            max = 10,
            step = 1,
            getFunc = function() return GDN.animationDuration / 1000 end,
            setFunc = function(value)
                local duration = value * 1000
                saveData.animationDuration = duration
                GDN.animationDuration = duration
                GDN.deathAnimation:SetDuration(duration)
                GDN.rezAnimation:SetDuration(duration)
            end,
            width = "full",
            default = 3,
        },
        [12] = {
            type = "slider",
            name = "Font Scale (Percentage)",
            tooltip = "Default = 100",
            min = 10,
            max = 200,
            step = 10,
            getFunc = function() return GDN.fontScale * 100 end,
            setFunc = function(value)
                local actualScale = value / 100
                saveData.fontScale = actualScale
                GDN.fontScale = actualScale
                GroupDeathIndicator:SetScale(actualScale)
                GroupRezIndicator:SetScale(actualScale)
                PetDeathIndicator:SetScale(actualScale)
            end,
            width = "full",
            default = 1,
        },
        [13] = {
            type = "colorpicker",
            name = "Death Notification Color",
            tooltip = "Choose color for death notification message",
            getFunc = function() 
                return GDN.deathNotificationColor["r"], GDN.deathNotificationColor["g"], GDN.deathNotificationColor["b"]
            end,
            setFunc = function(r,g,b,a) 
                GroupDeathLabel:SetColor(r,g,b)
                local deathColor = {
                    ["r"] = r,
                    ["g"] = g,
                    ["b"] = b,
                }
                GDN.deathNotificationColor = deathColor
                saveData.deathNotificationColor = deathColor
            end,
            width = "full",
        },
        [14] = {
            type = "colorpicker",
            name = "Rez Notification Color",
            tooltip = "Choose color for rez notification message",
            getFunc = function() 
                return GDN.rezNotificationColor["r"], GDN.rezNotificationColor["g"], GDN.rezNotificationColor["b"]  
            end,
            setFunc = function(r,g,b,a) 
                GroupRezLabel:SetColor(r,g,b)
                local rezColor = {
                    ["r"] = r,
                    ["g"] = g,
                    ["b"] = b,
                }
                GDN.rezNotificationColor = rezColor
                saveData.rezNotificationColor = rezColor
            end,
            width = "full",
        },
        [15] = {
            type = "colorpicker",
            name = "Pet Death Notification Color",
            tooltip = "Choose color for pet death notification message",
            getFunc = function() 
                return GDN.petNotificationColor["r"], GDN.petNotificationColor["g"], GDN.petNotificationColor["b"]
            end,
            setFunc = function(r,g,b,a) 
                PetDeathLabel:SetColor(r,g,b)
                local petColor = {
                    ["r"] = r,
                    ["g"] = g,
                    ["b"] = b,
                }
                GDN.petNotificationColor = petColor
                saveData.petNotificationColor = petColor
            end,
            width = "full",
        },
        [16] = {
            type = "checkbox",
            name = "UserID instead of Character Name",
            tooltip = "Turn on to show UserID instead of Character Name in notifications",
            getFunc = function() return GDN.useAccountName end,
            setFunc = function(value)
                GDN.useAccountName = value
                saveData.useAccountName = GDN.useAccountName
            end,
            width = "full",
        },
        [17] = {
            type = "checkbox",
            name = "Show Full Pet Name",
            tooltip = "Turn on to show full pet name in notification, otherwise it says Your Pet Died",
            getFunc = function() return GDN.fullPetName end,
            setFunc = function(value)
                GDN.fullPetName = value
                saveData.fullPetName = GDN.fullPetName
            end,
            width = "full",
        },
        [18] = {
            type = "header",
            name = "Dungeon Alert Settings",
            width = "full",	--or "half" (optional)
        },
        [19] = {
            type = "description",
            title = nil,	--(optional)
            text = "Show/hide notifications when your group size is 4 or less.",
            width = "full",	--or "half" (optional)
        },
        [20] = {
            type = "checkbox",
            name = "Tank Notifications",
            tooltip = "Turn off to hide Tank death notifications",
            getFunc = function() return GDN.dungeonSettings["Tank"] end,
            setFunc = function(value)
                GDN.dungeonSettings["Tank"] = value
                saveData.dungeonSettings = GDN.dungeonSettings
            end,
            width = "full",
        },
        [21] = {
            type = "checkbox",
            name = "Healer Notifications",
            tooltip = "Turn off to hide Healer death notifications",
            getFunc = function() return GDN.dungeonSettings["Healer"] end,
            setFunc = function(value)
                GDN.dungeonSettings["Healer"] = value
                saveData.dungeonSettings = GDN.dungeonSettings
            end,
            width = "full",
        },
        [22] = {
            type = "checkbox",
            name = "DPS Notifications",
            tooltip = "Turn off to hide DPS death notifications",
            getFunc = function() return GDN.dungeonSettings["DPS"] end,
            setFunc = function(value)
                GDN.dungeonSettings["DPS"] = value
                saveData.dungeonSettings = GDN.dungeonSettings
            end,
            width = "full",
        },
        [23] = {
            type = "checkbox",
            name = "Pet Notifications",
            tooltip = "Turn off to hide Pet death notifications",
            getFunc = function() return GDN.dungeonSettings["Pet"] end,
            setFunc = function(value)
                GDN.dungeonSettings["Pet"] = value
                saveData.dungeonSettings = GDN.dungeonSettings
            end,
            width = "full",
        },
        [24] = {
            type = "header",
            name = "Trial Alert Settings",
            width = "full",	--or "half" (optional)
        },
        [25] = {
            type = "description",
            title = nil,	--(optional)
            text = "Show/hide notifications when your group size is 5-12.",
            width = "full",	--or "half" (optional)
        },
        [26] = {
            type = "checkbox",
            name = "Tank Notifications",
            tooltip = "Turn off to hide Tank death notifications",
            getFunc = function() return GDN.trialSettings["Tank"] end,
            setFunc = function(value)
                GDN.trialSettings["Tank"] = value
                saveData.trialSettings = GDN.trialSettings
            end,
            width = "full",
        },
        [27] = {
            type = "checkbox",
            name = "Healer Notifications",
            tooltip = "Turn off to hide Healer death notifications",
            getFunc = function() return GDN.trialSettings["Healer"] end,
            setFunc = function(value)
                GDN.trialSettings["Healer"] = value
                saveData.trialSettings = GDN.trialSettings
            end,
            width = "full",
        },
        [28] = {
            type = "checkbox",
            name = "DPS Notifications",
            tooltip = "Turn off to hide DPS death notifications",
            getFunc = function() return GDN.trialSettings["DPS"] end,
            setFunc = function(value)
                GDN.trialSettings["DPS"] = value
                saveData.trialSettings = GDN.trialSettings
            end,
            width = "full",
        },
        [29] = {
            type = "checkbox",
            name = "Pet Notifications",
            tooltip = "Turn off to hide Pet death notifications",
            getFunc = function() return GDN.trialSettings["Pet"] end,
            setFunc = function(value)
                GDN.trialSettings["Pet"] = value
                saveData.trialSettings = GDN.trialSettings
            end,
            width = "full",
        },
        [30] = {
            type = "header",
            name = "Reset Defaults",
            width = "full",	--or "half" (optional)
        },
        [31] = {
            type = "description",
            title = nil,	--(optional)
            text = "Click button to RESET settings for this character back to DEFAULTS and RELOAD THE UI.",
            width = "full",	--or "half" (optional)
        },
        [32] = {
            type = "button",
            name = "Reset Defaults",
            tooltip = "Reset all settings to default and reload the UI",
            func = function() 
                GDN:ResetSavedValues()
                GDN:SetDefaultVars()
                ReloadUI()
            end,
            width = "full",	
            warning = "Will discard all settings and reload the UI!",
        },        
    }

    LAM:RegisterOptionControls(panelName, optionsData)
end